"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
const chai_1 = require("chai");
const cockatiel_1 = require("cockatiel");
const sinon_1 = require("sinon");
const __1 = require("..");
const connection_pool_1 = require("../connection-pool");
const errors_1 = require("../errors");
const util_1 = require("./util");
function getOptionsWithBadHost(options = {}) {
    return util_1.getOptions({
        hosts: [util_1.getHost(), '127.0.0.1:1'],
        ...options,
    });
}
describe('connection pool', () => {
    const key = Buffer.from('foo');
    const value = Buffer.from('bar');
    let pool;
    afterEach(() => {
        if (pool) {
            pool.close();
            pool = null;
        }
    });
    it('calls simple methods', async () => {
        pool = new connection_pool_1.ConnectionPool(util_1.getOptions());
        const kv = new __1.KVClient(pool);
        await kv.put({ key, value });
        const res = await kv.range({ key });
        chai_1.expect(res.kvs).to.containSubset([{ key, value }]);
        await kv.deleteRange({ key });
    });
    it('applies call options', async () => {
        const optsStub = sinon_1.stub()
            .onFirstCall()
            .returns({ deadline: new Date(0) })
            .onSecondCall()
            .returns({ deadline: new Date(Date.now() + 30000) });
        const pool = new connection_pool_1.ConnectionPool({ ...util_1.getOptions(), defaultCallOptions: optsStub });
        const kv = new __1.KVClient(pool);
        await chai_1.expect(kv.range({ key })).to.be.rejectedWith(errors_1.GRPCDeadlineExceededError);
        chai_1.expect(await kv.range({ key })).be.ok;
    });
    it('rejects instantiating with a mix of secure and unsecure hosts', () => {
        chai_1.expect(() => new connection_pool_1.ConnectionPool(util_1.getOptions({
            hosts: ['https://server1', 'http://server2'],
            credentials: undefined,
        }))).to.throw(/mix of secure and insecure hosts/);
    });
    it('rejects hitting invalid hosts', () => {
        pool = new connection_pool_1.ConnectionPool(getOptionsWithBadHost());
        const kv = new __1.KVClient(pool);
        return kv
            .range({ key })
            .then(() => {
            throw new Error('expected to reject');
        })
            .catch(err => chai_1.expect(err).to.be.an.instanceof(errors_1.GRPCUnavailableError));
    });
    it('should retry through policy', async () => {
        pool = new connection_pool_1.ConnectionPool(getOptionsWithBadHost({
            faultHandling: {
                global: cockatiel_1.Policy.handleAll().retry().attempts(3),
                host: () => cockatiel_1.Policy.noop,
            },
        }));
        const kv = new __1.KVClient(pool);
        chai_1.expect((await kv.range({ key })).kvs).to.deep.equal([]);
    });
});
//# sourceMappingURL=connection-pool.test.js.map