"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
const chai_1 = require("chai");
const util_1 = require("./util");
describe('crud', () => {
    let client;
    beforeEach(async () => (client = await util_1.createTestClientAndKeys()));
    afterEach(async () => await util_1.tearDownTestClient(client));
    describe('get() / getAll()', () => {
        it('lists all values', async () => {
            chai_1.expect(await client.getAll().strings()).to.deep.equal({
                foo1: 'bar1',
                foo2: 'bar2',
                foo3: '{"value":"bar3"}',
                baz: 'bar5',
            });
        });
        it('gets single keys with various encoding', async () => {
            chai_1.expect(await client.get('foo1').string()).to.equal('bar1');
            chai_1.expect(await client.get('foo2').buffer()).to.deep.equal(Buffer.from('bar2'));
            chai_1.expect(await client.get('foo3').json()).to.deep.equal({ value: 'bar3' });
            chai_1.expect(await client.get('wut').string()).to.be.null;
        });
        it('gets whether keys exist', async () => {
            chai_1.expect(await client.get('foo1').exists()).to.be.true;
            chai_1.expect(await client.get('wut').exists()).to.be.false;
        });
        it('queries prefixes', async () => {
            chai_1.expect(await client.getAll().prefix('fo').strings()).to.deep.equal({
                foo1: 'bar1',
                foo2: 'bar2',
                foo3: '{"value":"bar3"}',
            });
        });
        it('gets keys', async () => {
            chai_1.expect(await client.getAll().keys()).to.have.members(['foo1', 'foo2', 'foo3', 'baz']);
            chai_1.expect(await client.getAll().keyBuffers()).to.have.deep.members([
                Buffer.from('foo1'),
                Buffer.from('foo2'),
                Buffer.from('foo3'),
                Buffer.from('baz'),
            ]);
        });
        it('counts', async () => {
            chai_1.expect(await client.getAll().count()).to.equal(4);
        });
        it('sorts', async () => {
            chai_1.expect(await client.getAll().prefix('foo').sort('Key', 'Ascend').limit(2).keys()).to.deep.equal(['foo1', 'foo2']);
            chai_1.expect(await client.getAll().prefix('foo').sort('Key', 'Descend').limit(2).keys()).to.deep.equal(['foo3', 'foo2']);
        });
    });
    describe('delete()', () => {
        it('deletes all', async () => {
            await client.delete().all();
            chai_1.expect(await client.getAll().count()).to.equal(0);
        });
        it('deletes prefix', async () => {
            await client.delete().prefix('foo');
            chai_1.expect(await client.getAll().keys()).to.deep.equal(['baz']);
        });
        it('gets previous', async () => {
            chai_1.expect(await client.delete().key('foo1').getPrevious()).to.containSubset([
                {
                    key: Buffer.from('foo1'),
                    value: Buffer.from('bar1'),
                },
            ]);
        });
        describe('put()', () => {
            it('allows touching key revisions', async () => {
                const original = (await client.get('foo1').exec()).kvs[0].mod_revision;
                await client.put('foo1').touch();
                const updated = (await client.get('foo1').exec()).kvs[0].mod_revision;
                chai_1.expect(Number(updated)).to.be.greaterThan(Number(original));
            });
            it('updates key values', async () => {
                await client.put('foo1').value('updated');
                chai_1.expect(await client.get('foo1').string()).to.equal('updated');
            });
            it('includes previous values', async () => {
                chai_1.expect(await client.put('foo1').value('updated').getPrevious()).to.containSubset({
                    key: Buffer.from('foo1'),
                    value: Buffer.from('bar1'),
                });
            });
        });
    });
});
//# sourceMappingURL=crud.test.js.map