"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
const chai_1 = require("chai");
const sinon = require("sinon");
const __1 = require("..");
const util_1 = require("../util");
const util_2 = require("./util");
const errors_1 = require("../errors");
describe('lease()', () => {
    let client;
    let lease;
    let clock;
    beforeEach(async () => (client = await util_2.createTestClientAndKeys()));
    afterEach(async () => {
        if (lease && !lease.revoked()) {
            await lease.revoke();
        }
        await util_2.tearDownTestClient(client);
        if (clock) {
            clock.restore();
            clock = undefined;
        }
    });
    const watchEmission = (event) => {
        const output = { data: null, fired: false };
        lease.once(event, (data) => {
            output.data = data;
            output.fired = true;
        });
        return output;
    };
    it('throws if trying to use too short of a ttl, or an undefined ttl', () => {
        chai_1.expect(() => client.lease(0)).to.throw(/must be at least 1 second/);
        chai_1.expect(() => client.lease()).to.throw(/must be at least 1 second/);
    });
    it('reports a loss and errors if the client is invalid', async () => {
        const badClient = new __1.Etcd3(util_2.getOptions({ hosts: '127.0.0.1:1' }));
        lease = badClient.lease(1);
        const err = await util_1.onceEvent(lease, 'lost');
        chai_1.expect(err).to.be.an.instanceof(errors_1.GRPCUnavailableError);
        await lease
            .grant()
            .then(() => {
            throw new Error('expected to reject');
        })
            .catch(err2 => chai_1.expect(err2).to.equal(err));
        badClient.close();
    });
    it('provides basic lease lifecycle', async () => {
        lease = client.lease(100);
        await lease.put('leased').value('foo');
        chai_1.expect((await client.get('leased').exec()).kvs[0].lease).to.equal(await lease.grant());
        await lease.revoke();
        chai_1.expect(await client.get('leased').buffer()).to.be.null;
    });
    it('attaches leases through transactions', async () => {
        lease = client.lease(100);
        await lease.put('leased').value('foo');
        const result = await client
            .if('foo1', 'Value', '==', 'bar1')
            .then(lease.put('leased').value('foo'))
            .commit();
        chai_1.expect(result.succeeded).to.equal(true, 'expected to have completed transaction');
        chai_1.expect((await client.get('leased').exec()).kvs[0].lease).to.equal(await lease.grant());
        await lease.revoke();
        chai_1.expect(await client.get('leased').buffer()).to.be.null;
    });
    it('runs immediate keepalives', async () => {
        lease = client.lease(100);
        chai_1.expect(await lease.keepaliveOnce()).to.containSubset({
            ID: await lease.grant(),
            TTL: '100',
        });
        await lease.keepaliveOnce();
    });
    it('is resilient to network interruptions', async () => {
        await util_2.proxy.activate();
        const proxiedClient = new __1.Etcd3(util_2.getOptions());
        lease = proxiedClient.lease(100);
        await lease.grant();
        util_2.proxy.suspend();
        await util_1.onceEvent(lease, 'keepaliveFailed');
        util_2.proxy.unsuspend();
        await util_1.onceEvent(lease, 'keepaliveSucceeded');
        await lease.revoke();
        proxiedClient.close();
        await util_2.proxy.deactivate();
    });
    it('marks leases as failed if the server is not contacted for a while', async () => {
        await util_2.proxy.activate();
        const proxiedClient = new __1.Etcd3(util_2.getOptions());
        lease = proxiedClient.lease(1);
        await lease.grant();
        util_2.proxy.suspend();
        lease.lastKeepAlive = Date.now() - 2000; // speed things up a little
        const err = await util_1.onceEvent(lease, 'lost');
        chai_1.expect(err.message).to.match(/our lease has expired/);
        proxiedClient.close();
        await util_2.proxy.deactivate();
    });
    it('emits a lost event if the lease is invalidated', async () => {
        lease = client.lease(100);
        let err;
        lease.on('lost', e => {
            chai_1.expect(lease.revoked()).to.be.true;
            err = e;
        });
        chai_1.expect(lease.revoked()).to.be.false;
        await client.leaseClient.leaseRevoke({ ID: await lease.grant() });
        await lease
            .keepaliveOnce()
            .then(() => {
            throw new Error('expected to reject');
        })
            .catch(err2 => {
            chai_1.expect(err2).to.equal(err);
            chai_1.expect(err2).to.be.an.instanceof(__1.EtcdLeaseInvalidError);
            chai_1.expect(lease.revoked()).to.be.true;
        });
    });
    it('emits a loss if the touched key is lost', async () => {
        lease = client.lease(100, { autoKeepAlive: false });
        lease.leaseID = Promise.resolve('123456789');
        const lost = util_1.onceEvent(lease, 'lost');
        try {
            await lease.put('foo').value('bar');
        }
        catch (e) {
            chai_1.expect(e).to.be.an.instanceof(__1.EtcdLeaseInvalidError);
            chai_1.expect(e).to.equal(await lost);
            chai_1.expect(lease.revoked()).to.be.true;
        }
    });
    it('allows disabling auto keep alives', async () => {
        clock = sinon.useFakeTimers({
            shouldAdvanceTime: true,
        });
        lease = client.lease(60, { autoKeepAlive: false });
        const kaFired = watchEmission('keepaliveFired');
        clock.tick(20000);
        chai_1.expect(kaFired.fired).to.be.false;
    });
    describe('crons', () => {
        beforeEach(async () => {
            clock = sinon.useFakeTimers({
                shouldAdvanceTime: true,
            });
            lease = client.lease(60);
            await util_1.onceEvent(lease, 'keepaliveEstablished');
        });
        it('touches the lease ttl at the correct interval', async () => {
            const kaFired = watchEmission('keepaliveFired');
            clock.tick(19999);
            chai_1.expect(kaFired.fired).to.be.false;
            clock.tick(1);
            chai_1.expect(kaFired.fired).to.be.true;
            const res = await util_1.onceEvent(lease, 'keepaliveSucceeded');
            chai_1.expect(res.TTL).to.equal('60');
        });
        it('stops touching the lease if released passively', async () => {
            const kaFired = watchEmission('keepaliveFired');
            lease.release();
            clock.tick(20000);
            chai_1.expect(kaFired.fired).to.be.false;
        });
        it('marks leases as failed if etcd does not respond to keepalives in time (#110)', async () => {
            await lease.revoke();
            await util_2.proxy.activate();
            const proxiedClient = new __1.Etcd3(util_2.getOptions());
            lease = proxiedClient.lease(1);
            await lease.grant();
            util_2.proxy.pause(1 /* FromEtcd */);
            const failedEvent = watchEmission('keepaliveFailed');
            clock.tick(50000);
            await util_2.unmockedDelay(2); // drain task queues
            chai_1.expect(failedEvent.fired).to.be.false;
            clock.tick(10000);
            await util_2.unmockedDelay(2); // drain task queues
            chai_1.expect(failedEvent.fired).to.be.true;
            util_2.proxy.resume(1 /* FromEtcd */);
            await lease.revoke();
            proxiedClient.close();
            util_2.proxy.deactivate();
        });
        it('tears down if the lease gets revoked', async () => {
            await client.leaseClient.leaseRevoke({ ID: await lease.grant() });
            clock.tick(20000);
            chai_1.expect(await util_1.onceEvent(lease, 'lost')).to.be.an.instanceof(__1.EtcdLeaseInvalidError);
            chai_1.expect(lease.revoked()).to.be.true;
        });
    });
});
//# sourceMappingURL=lease.test.js.map