"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
const chai_1 = require("chai");
const __1 = require("..");
const util_1 = require("./util");
describe('lock()', () => {
    let client;
    beforeEach(async () => (client = await util_1.createTestClientAndKeys()));
    afterEach(async () => await util_1.tearDownTestClient(client));
    const assertCantLock = () => {
        return chai_1.expect(client.lock('resource').acquire()).to.eventually.be.rejectedWith(__1.EtcdLockFailedError);
    };
    const assertAbleToLock = async () => {
        const lock = client.lock('resource');
        await lock.acquire();
        await lock.release();
    };
    it('locks exclusively around a resource', async () => {
        const lock1 = client.lock('resource');
        await lock1.acquire();
        await assertCantLock();
        await lock1.release();
        await assertAbleToLock();
    });
    it('provides locking around functions', async () => {
        await client.lock('resource').do(assertCantLock);
        await assertAbleToLock();
    });
    it('allows setting lock TTL before acquiring', async () => {
        const lock = await client.lock('resource').ttl(10).acquire();
        await lock.release();
    });
    it('disallows setting TTL while lock is acquired', async () => {
        const lock = await client.lock('resource').acquire();
        chai_1.expect(() => lock.ttl(10)).to.throw(/Cannot set a lock TTL after acquiring the lock/);
        await lock.release();
    });
    it('gets the lock lease ID', async () => {
        const lock = await client.lock('resource');
        chai_1.expect(await lock.leaseId()).to.equal(null, 'expected no lease initially');
        await lock.acquire();
        const leaseId = await lock.leaseId();
        chai_1.expect(leaseId).to.be.a('string');
        chai_1.expect((await client.get('resource').exec()).kvs[0].lease).to.equal(leaseId);
        await lock.release();
    });
});
//# sourceMappingURL=lock.test.js.map