"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
const chai_1 = require("chai");
const grpc = require("@grpc/grpc-js");
const __1 = require("..");
const util_1 = require("./util");
const errors_1 = require("../errors");
function wipeAll(things) {
    return things.then(items => Promise.all(items.map(item => item.delete())));
}
describe('roles and auth', () => {
    let client;
    beforeEach(async () => (client = await util_1.createTestClientAndKeys()));
    afterEach(async () => await util_1.tearDownTestClient(client));
    describe('management', () => {
        afterEach(() => wipeAll(client.getRoles()));
        const expectRoles = async (expected) => {
            const list = await client.getRoles();
            chai_1.expect(list.map(r => r.name)).to.deep.equal(expected);
        };
        it('create and deletes', async () => {
            const fooRole = await client.role('foo').create();
            await expectRoles(['foo']);
            await fooRole.delete();
            await expectRoles([]);
        });
        it('throws on existing roles', async () => {
            await client.role('foo').create();
            await util_1.expectReject(client.role('foo').create(), __1.EtcdRoleExistsError);
        });
        it('throws on deleting a non-existent role', async () => {
            await util_1.expectReject(client.role('foo').delete(), __1.EtcdRoleNotFoundError);
        });
        it('throws on granting permission to a non-existent role', async () => {
            await util_1.expectReject(client.role('foo').grant({
                permission: 'Read',
                range: client.range({ prefix: '111' }),
            }), __1.EtcdRoleNotFoundError);
        });
        it('round trips permission grants', async () => {
            const fooRole = await client.role('foo').create();
            await fooRole.grant({
                permission: 'Read',
                range: client.range({ prefix: '111' }),
            });
            const perms = await fooRole.permissions();
            chai_1.expect(perms).to.containSubset([
                {
                    permission: 'Read',
                    range: client.range({ prefix: '111' }),
                },
            ]);
            await fooRole.revoke(perms[0]);
            chai_1.expect(await fooRole.permissions()).to.have.length(0);
        });
    });
    describe('users', () => {
        let fooRole;
        beforeEach(async () => {
            fooRole = client.role('foo');
            await fooRole.create();
        });
        afterEach(async () => {
            await fooRole.delete();
            await wipeAll(client.getUsers());
        });
        it('creates users', async () => {
            chai_1.expect(await client.getUsers()).to.have.lengthOf(0);
            await client.user('connor').create('password');
            chai_1.expect(await client.getUsers()).to.containSubset([{ name: 'connor' }]);
        });
        it('throws on existing users', async () => {
            await client.user('connor').create('password');
            await util_1.expectReject(client.user('connor').create('password'), __1.EtcdUserExistsError);
        });
        it('throws on regranting the same role multiple times', async () => {
            const user = await client.user('connor').create('password');
            await util_1.expectReject(user.removeRole(fooRole), __1.EtcdRoleNotGrantedError);
        });
        it('throws on granting a non-existent role', async () => {
            const user = await client.user('connor').create('password');
            await util_1.expectReject(user.addRole('wut'), __1.EtcdRoleNotFoundError);
        });
        it('throws on deleting a non-existent user', async () => {
            await util_1.expectReject(client.user('connor').delete(), __1.EtcdUserNotFoundError);
        });
        it('round trips roles', async () => {
            const user = await client.user('connor').create('password');
            await user.addRole(fooRole);
            chai_1.expect(await user.roles()).to.containSubset([{ name: 'foo' }]);
            await user.removeRole(fooRole);
            chai_1.expect(await user.roles()).to.have.lengthOf(0);
        });
    });
    describe('password auth', () => {
        beforeEach(async () => {
            await util_1.setupAuth(client);
        });
        afterEach(async () => {
            await util_1.removeAuth(client);
        });
        it('allows authentication using the correct credentials', async () => {
            const authedClient = new __1.Etcd3(util_1.getOptions({
                auth: {
                    username: 'connor',
                    password: 'password',
                },
            }));
            await authedClient.put('foo').value('bar');
            authedClient.close();
        });
        it('applies call options', async () => {
            const authedClient = new __1.Etcd3(util_1.getOptions({
                auth: {
                    username: 'connor',
                    password: 'password',
                    callOptions: { deadline: new Date(0) },
                },
            }));
            await chai_1.expect(authedClient.put('foo').value('bar')).to.be.rejectedWith(errors_1.GRPCDeadlineExceededError);
            authedClient.close();
        });
        it('rejects modifying a key the client has no access to', async () => {
            const authedClient = new __1.Etcd3(util_1.getOptions({
                auth: {
                    username: 'connor',
                    password: 'password',
                },
            }));
            await util_1.expectReject(authedClient.put('wut').value('bar').exec(), __1.EtcdPermissionDeniedError);
            authedClient.close();
        });
        it('throws when using incorrect credentials', async () => {
            const authedClient = new __1.Etcd3(util_1.getOptions({
                auth: {
                    username: 'connor',
                    password: 'bad password',
                },
            }));
            await util_1.expectReject(authedClient.put('foo').value('bar').exec(), __1.EtcdAuthenticationFailedError);
            authedClient.close();
        });
        it('automatically retrieves a new token if the existing one is invalid', async () => {
            const authedClient = new __1.Etcd3(util_1.getOptions({
                auth: {
                    username: 'connor',
                    password: 'password',
                },
            }));
            const auth = authedClient.pool.authenticator;
            const badMeta = new grpc.Metadata();
            badMeta.add('token', 'lol');
            auth.awaitingMetadata = Promise.resolve(badMeta);
            await authedClient.put('foo').value('bar'); // should retry and not throw
            authedClient.close();
            const updatedMeta = await auth.awaitingMetadata;
            chai_1.expect(updatedMeta.get('token')).to.not.deep.equal(badMeta.get('token'));
        });
    });
});
//# sourceMappingURL=roles.test.js.map