"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
const chai_1 = require("chai");
const __1 = require("..");
const util_1 = require("./util");
describe('stm()', () => {
    [
        {
            namespace: false,
            name: 'without namespace',
        },
        {
            namespace: true,
            name: 'with namespace',
        },
    ].forEach(testcase => describe(testcase.name, () => {
        let client;
        let ns;
        beforeEach(async () => {
            client = await util_1.createTestClient();
            ns = testcase.namespace ? client.namespace('ns/') : client;
            await util_1.createTestKeys(ns);
        });
        afterEach(async () => await util_1.tearDownTestClient(client));
        it('executes empty transactions', async () => {
            chai_1.expect(await ns.stm().transact(() => 'foo')).to.equal('foo');
        });
        const expectRetry = async (isolation, fn, retries = 2) => {
            let tries = 0;
            await ns.stm({ isolation }).transact(async (tx) => fn(tx, ++tries));
            chai_1.expect(tries).to.equal(retries);
        };
        const expectRunsCleanTransaction = (isolation) => {
            it('runs transactions when all is good', async () => {
                await ns.stm({ isolation }).transact(async (tx) => {
                    const value = await tx.get('foo1');
                    await tx.put('foo1').value(value.repeat(3));
                    chai_1.expect(await ns.get('foo1')).to.equal('bar1'); // should not have changed yet
                });
                chai_1.expect(await ns.get('foo1')).to.equal('bar1bar1bar1');
            });
        };
        const expectRepeatableReads = (isolation) => {
            it('has repeatable reads on existing keys', async () => {
                await expectRetry(isolation, async (tx, tries) => {
                    await tx.get('foo1');
                    if (tries === 1) {
                        // should fail when the key changes before the transaction commits
                        await ns.put('foo1').value('lol');
                    }
                });
            });
            it('has repeatable reads on non-existent', async () => {
                await expectRetry(isolation, async (tx, tries) => {
                    await tx.get('some-key-that-does-not-exist');
                    if (tries === 1) {
                        await ns.put('some-key-that-does-not-exist').value('lol');
                    }
                });
            });
        };
        const ignoreConflicts = (isolation, fn) => {
            return ns
                .stm({ retries: 0, isolation })
                .transact(fn)
                .catch(err => {
                if (!(err instanceof __1.STMConflictError)) {
                    throw err;
                }
            });
        };
        const expectWriteCaching = (isolation) => {
            it('caches writes in memory (#1)', () => {
                return ignoreConflicts(isolation, async (tx) => {
                    // putting and value and getting it should returned the value to be written
                    await tx.put('foo').value('some value');
                    chai_1.expect(await tx.get('foo').string()).to.equal('some value');
                });
            });
            it('caches writes in memory (#2)', async () => {
                return ignoreConflicts(isolation, async (tx) => {
                    // getting a value, then overwriting it, should return the overwritten value
                    chai_1.expect(await tx.get('foo1').string()).to.equal('bar1');
                    await tx.put('foo1').value('lol');
                    chai_1.expect(await tx.get('foo1').string()).to.equal('lol');
                });
            });
            it('caches writes in memory (#3)', async () => {
                return ignoreConflicts(isolation, async (tx) => {
                    // deleting a value should null it
                    await tx.delete().key('foo1');
                    chai_1.expect(await tx.get('foo1').string()).to.be.null;
                    // subsequently writing a key should put it back
                    await tx.put('foo1').value('lol');
                    chai_1.expect(await tx.get('foo1').string()).to.equal('lol');
                });
            });
            it('caches writes in memory (#4)', async () => {
                return ignoreConflicts(isolation, async (tx) => {
                    // deleting a range should null all keys in that range
                    await tx.delete().prefix('foo');
                    chai_1.expect(await tx.get('foo2').string()).to.be.null;
                });
            });
        };
        const expectReadCaching = (isolation) => {
            it('caches reads in memory', async () => {
                return ns
                    .stm({ retries: 0, isolation })
                    .transact(async (tx) => {
                    chai_1.expect(await tx.get('foo1').string()).to.equal('bar1');
                    await ns.put('foo1').value('changed!');
                    chai_1.expect(await tx.get('foo1').string()).to.equal('bar1');
                })
                    .catch(() => undefined);
            });
        };
        describe('ReadCommitted', () => {
            expectWriteCaching(3 /* ReadCommitted */);
            expectRunsCleanTransaction(3 /* ReadCommitted */);
        });
        describe('RepeatableReads', () => {
            expectWriteCaching(2 /* RepeatableReads */);
            expectRunsCleanTransaction(2 /* RepeatableReads */);
            expectRepeatableReads(2 /* RepeatableReads */);
        });
        describe('Serializable', () => {
            expectWriteCaching(1 /* Serializable */);
            expectRunsCleanTransaction(1 /* Serializable */);
            expectRepeatableReads(1 /* Serializable */);
            expectReadCaching(1 /* Serializable */);
        });
        describe('SerializableSnapshot', () => {
            expectWriteCaching(0 /* SerializableSnapshot */);
            expectRunsCleanTransaction(0 /* SerializableSnapshot */);
            expectRepeatableReads(0 /* SerializableSnapshot */);
            expectReadCaching(0 /* SerializableSnapshot */);
            it('should deny writing ranges if keys are read', () => {
                return chai_1.expect(ignoreConflicts(0 /* SerializableSnapshot */, async (tx) => {
                    await tx.get('foo1').string();
                    await tx.delete().prefix('foo');
                })).to.eventually.be.rejectedWith(/You cannot delete ranges/);
            });
            // the blueprint for the next two is:
            // 1. get foo1
            // 2. outside the transaction, set it to something else
            // 3. try to writed/delete it and fail
            // 4. get foo1, this time write it succesfully to what was set before
            it('retries writes on conflicts', async () => {
                await expectRetry(0 /* SerializableSnapshot */, async (tx, tries) => {
                    const value = await tx.get('foo1');
                    if (tries === 1) {
                        await ns.put('foo1').value('lol');
                    }
                    await tx.put('foo1').value(value.repeat(3));
                });
                chai_1.expect(await ns.get('foo1')).to.equal('lollollol');
            });
            it('retries deletes on conflicts', async () => {
                await expectRetry(0 /* SerializableSnapshot */, async (tx, tries) => {
                    await tx.get('foo1');
                    if (tries === 1) {
                        await ns.put('foo1').value('lol');
                    }
                    await tx.delete().key('foo1');
                });
                chai_1.expect(await ns.get('foo1')).to.null;
            });
            it('aborts transactions on continous failure', async () => {
                await chai_1.expect(ns
                    .stm({ isolation: 0 /* SerializableSnapshot */ })
                    .transact(async (tx) => {
                    const value = await tx.get('foo1');
                    await ns.put('foo1').value('lol');
                    await tx.put('foo1').value(value.repeat(3));
                })
                    .then(() => {
                    throw new Error('expected to throw');
                })).to.eventually.be.rejectedWith(__1.STMConflictError);
            });
        });
    }));
});
//# sourceMappingURL=stm.test.js.map