"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.unmockedDelay = exports.atAtMostVersion = exports.isAtLeastVersion = exports.removeAuth = exports.setupAuth = exports.tearDownTestClient = exports.createTestKeys = exports.createTestClientAndKeys = exports.createTestClient = exports.expectReject = exports.getOptions = exports.getHost = exports.proxy = exports.Proxy = exports.etcdVersion = void 0;
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
const chai_1 = require("chai");
const fs = require("fs");
const tls = require("tls");
const __1 = require("..");
const path_1 = require("path");
const cockatiel_1 = require("cockatiel");
const rootPath = path_1.resolve(__dirname, '..', '..');
const rootCertificate = fs.readFileSync(`${rootPath}/src/test/certs/certs/ca.crt`);
const tlsCert = fs.readFileSync(`${rootPath}/src/test/certs/certs/etcd0.localhost.crt`);
const tlsKey = fs.readFileSync(`${rootPath}/src/test/certs/private/etcd0.localhost.key`);
const etcdSourceAddress = process.env.ETCD_ADDR || '127.0.0.1:2379';
const [etcdSourceHost, etcdSourcePort] = etcdSourceAddress.split(':');
exports.etcdVersion = process.env.ETCD_VERSION || '3.3.9';
/**
 * Proxy is a TCP proxy for etcd, used so that we can simulate network failures
 * and disruptions in a cross-platform manner (i.e no reliance on tcpkill
 * or ip link)
 */
class Proxy {
    constructor() {
        this.isActive = false;
        this.connections = [];
        this.enabledDataFlows = new Set([1 /* FromEtcd */, 0 /* ToEtcd */]);
    }
    /**
     * activate creates the proxy server.
     */
    activate() {
        this.enabledDataFlows.add(1 /* FromEtcd */);
        this.enabledDataFlows.add(0 /* ToEtcd */);
        return new Promise((resolve, reject) => {
            this.server = tls.createServer({ cert: tlsCert, key: tlsKey, ALPNProtocols: ['h2'] }, clientCnx => this.handleIncoming(clientCnx));
            this.server.listen(0, '127.0.0.1');
            this.server.on('listening', () => {
                const addr = this.server.address();
                this.host = addr.address;
                this.port = addr.port;
                this.isActive = true;
                resolve();
            });
            this.server.on('error', err => {
                reject(err);
                console.error('error setting up server', err);
            });
        });
    }
    /**
     * suspend temporarily shuts down the server, but does not 'deactivate' the
     * proxy; new connections will still try to hit it. Can be restored with
     * resume().
     */
    suspend() {
        this.connections.forEach(cnx => cnx.end());
        this.server.close();
    }
    /**
     * Starts up a previously stopped server.
     */
    unsuspend() {
        this.server.listen(this.port, this.host);
    }
    /**
     * Disables data flowing in one direction on the connection.
     */
    pause(direction) {
        this.enabledDataFlows.delete(direction);
    }
    /**
     * Reenables data flow on the connection.
     */
    resume(direction) {
        this.enabledDataFlows.add(direction);
    }
    /**
     * Destroys a previously-active proxy server.
     */
    async deactivate() {
        this.isActive = false;
        await new Promise(r => this.server.close(r));
    }
    /**
     * Returns the address the server is listening on.
     */
    address() {
        return `${this.host}:${this.port}`;
    }
    handleIncoming(clientCnx) {
        let serverConnected = false;
        const serverBuffer = [];
        const serverCnx = tls.connect(Number(etcdSourcePort), etcdSourceHost, {
            secureContext: tls.createSecureContext({ ca: rootCertificate }),
            ALPNProtocols: ['h2'],
        }, () => {
            if (serverBuffer.length > 0 && !ended) {
                serverCnx.write(Buffer.concat(serverBuffer));
            }
            serverConnected = true;
        });
        let ended = false;
        const end = (err) => {
            ended = true;
            if (err instanceof Error) {
                throw err;
            }
            clientCnx.end();
            serverCnx.end();
            this.connections = this.connections.filter(c => c.end !== end);
        };
        serverCnx.on('data', (data) => {
            if (ended || !this.enabledDataFlows.has(1 /* FromEtcd */)) {
                return;
            }
            clientCnx.write(data);
        });
        serverCnx.on('end', end);
        serverCnx.on('error', end);
        clientCnx.on('data', (data) => {
            if (ended || !this.enabledDataFlows.has(0 /* ToEtcd */)) {
                return;
            }
            if (serverConnected) {
                serverCnx.write(data);
            }
            else {
                serverBuffer.push(data);
            }
        });
        clientCnx.on('end', end);
        clientCnx.on('error', end);
        this.connections.push({ end });
    }
}
exports.Proxy = Proxy;
exports.proxy = new Proxy();
/**
 * Returns the host to test against.
 */
function getHost() {
    if (exports.proxy.isActive) {
        return exports.proxy.address();
    }
    return process.env.ETCD_ADDR || '127.0.0.1:2379';
}
exports.getHost = getHost;
/**
 * Returns etcd options to use for connections.
 */
function getOptions(defaults = {}) {
    return {
        hosts: getHost(),
        credentials: { rootCertificate },
        faultHandling: {
            global: cockatiel_1.Policy.noop,
            host: () => cockatiel_1.Policy.noop,
        },
        ...defaults,
    };
}
exports.getOptions = getOptions;
/**
 * Returns a promise that throws if the promise is resolved or rejected with
 * something other than the provided constructor
 */
function expectReject(promise, err) {
    return promise
        .then(() => {
        throw new Error('expected to reject');
    })
        .catch(actualErr => {
        if (!(actualErr instanceof err)) {
            // tslint:disable-next-line
            console.error(actualErr.stack);
            chai_1.expect(actualErr).to.be.an.instanceof(err);
        }
    });
}
exports.expectReject = expectReject;
/**
 * Creates a new test etcd client.
 */
function createTestClient() {
    return new __1.Etcd3(getOptions());
}
exports.createTestClient = createTestClient;
/**
 * Creates an etcd client with the default options and seeds some keys.
 */
async function createTestClientAndKeys() {
    const client = createTestClient();
    await createTestKeys(client);
    return client;
}
exports.createTestClientAndKeys = createTestClientAndKeys;
/**
 * Creates test keys in the given namespace.
 */
async function createTestKeys(client) {
    await Promise.all([
        client.put('foo1').value('bar1'),
        client.put('foo2').value('bar2'),
        client.put('foo3').value('{"value":"bar3"}'),
        client.put('baz').value('bar5'),
    ]);
}
exports.createTestKeys = createTestKeys;
/**
 * Destroys the etcd client and wipes all keys.
 */
async function tearDownTestClient(client) {
    await (client === null || client === void 0 ? void 0 : client.delete().all());
    client.close();
}
exports.tearDownTestClient = tearDownTestClient;
function wipeAll(things) {
    return things.then(items => Promise.all(items.map(item => item.delete())));
}
/**
 * Sets up authentication for the server.
 */
async function setupAuth(client) {
    await wipeAll(client.getUsers());
    await wipeAll(client.getRoles());
    // We need to set up a root user and root role first, otherwise etcd
    // will yell at us.
    const rootUser = await client.user('root').create('password');
    await rootUser.addRole('root');
    await client.user('connor').create('password');
    const normalRole = await client.role('rw_prefix_f').create();
    await normalRole.grant({
        permission: 'Readwrite',
        range: client.range({ prefix: 'f' }),
    });
    await normalRole.addUser('connor');
    await client.auth.authEnable();
}
exports.setupAuth = setupAuth;
/**
 * Removes authentication previously added with `setupAuth`
 */
async function removeAuth(client) {
    const rootClient = new __1.Etcd3(getOptions({
        auth: {
            username: 'root',
            password: 'password',
        },
    }));
    await rootClient.auth.authDisable();
    rootClient.close();
    await wipeAll(client.getUsers());
    await wipeAll(client.getRoles());
}
exports.removeAuth = removeAuth;
const compareVersion = (version) => {
    var _a;
    const aParts = exports.etcdVersion.split('.').map(Number);
    const bParts = version.split('.').map(Number);
    return (_a = aParts.map((a, i) => a - bParts[i]).find(cmp => cmp !== 0)) !== null && _a !== void 0 ? _a : 0;
};
exports.isAtLeastVersion = (version) => compareVersion(version) >= 0;
exports.atAtMostVersion = (version) => compareVersion(version) <= 0;
const originalSetTimeout = setTimeout;
exports.unmockedDelay = (duration) => new Promise(r => originalSetTimeout(r, duration));
//# sourceMappingURL=util.js.map