"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
const bignumber_js_1 = require("bignumber.js");
const chai_1 = require("chai");
const __1 = require("..");
const util_1 = require("../util");
const util_2 = require("./util");
const errors_1 = require("../errors");
describe('watch()', () => {
    let client;
    beforeEach(async () => {
        client = new __1.Etcd3(util_2.getOptions());
    });
    afterEach(async () => {
        await util_2.tearDownTestClient(client);
    });
    /**
     * Returns the list of watchers currently attached and listening.
     */
    function getWatchers() {
        return client.watchManager.watchers;
    }
    /**
     * Checks that the watcher is getting updates for the given key.
     */
    function expectWatching(watcher, key) {
        return Promise.all([
            client.put(key).value('updated!'),
            util_1.onceEvent(watcher, 'put').then((res) => {
                chai_1.expect(res.key.toString()).to.equal(key);
                chai_1.expect(res.value.toString()).to.equal('updated!');
            }),
        ]).then(() => watcher);
    }
    /**
     * Checks that the watcher is not getting updates for the given key.
     */
    async function expectNotWatching(watcher, key) {
        let watching = false;
        const listener = () => (watching = true);
        watcher.on('put', listener);
        await client.put(key).value('updated!');
        return new Promise(resolve => {
            setTimeout(() => {
                chai_1.expect(watching).to.equal(false, `expected not to be watching ${key}`);
                resolve(watcher);
            }, 200);
        });
    }
    describe('network interruptions', () => {
        it.skip('is resilient to network interruptions', async () => {
            await util_2.proxy.activate();
            const proxiedClient = await util_2.createTestClientAndKeys();
            const watcher = await proxiedClient.watch().key('foo1').create();
            util_2.proxy.suspend();
            await util_1.onceEvent(watcher, 'disconnected');
            util_2.proxy.unsuspend();
            await util_1.onceEvent(watcher, 'connected');
            await expectWatching(watcher, 'foo1');
            await watcher.cancel();
            proxiedClient.close();
            await util_2.proxy.deactivate();
        });
        // todo(connor4312): this is disabled pending resolution on:
        // https://github.com/grpc/grpc-node/issues/80
        it.skip('replays historical updates', async () => {
            await util_2.proxy.activate();
            const proxiedClient = await util_2.createTestClientAndKeys();
            const watcher = await proxiedClient.watch().key('foo1').create();
            await Promise.all([
                client.put('foo1').value('update 1'),
                util_1.onceEvent(watcher, 'data').then((res) => {
                    chai_1.expect(watcher.request.start_revision).to.equal(new bignumber_js_1.default(res.header.revision).plus(1).toString());
                }),
            ]);
            util_2.proxy.suspend();
            await util_1.onceEvent(watcher, 'disconnected');
            util_2.proxy.unsuspend();
            await util_1.onceEvent(watcher, 'put').then((res) => {
                chai_1.expect(res.key.toString()).to.equal('foo1');
                chai_1.expect(res.value.toString()).to.equal('update 2');
            });
            await watcher.cancel();
            proxiedClient.close();
            await util_2.proxy.deactivate();
        });
        it('caps watchers revisions', async () => {
            await util_2.proxy.activate();
            const proxiedClient = await util_2.createTestClientAndKeys();
            const watcher = await proxiedClient.watch().key('foo1').create();
            util_2.proxy.suspend();
            await util_1.onceEvent(watcher, 'disconnected');
            const actualRevision = Number(watcher.request.start_revision);
            watcher.request.start_revision = 999999;
            util_2.proxy.unsuspend();
            await util_1.onceEvent(watcher, 'connected');
            chai_1.expect(Number(watcher.request.start_revision)).to.equal(actualRevision);
            await watcher.cancel();
            proxiedClient.close();
            await util_2.proxy.deactivate();
        });
        describe('emits an error if a watcher is cancelled upon creation (#114)', () => {
            beforeEach(() => util_2.setupAuth(client));
            afterEach(() => util_2.removeAuth(client));
            if (util_2.isAtLeastVersion('3.2.0')) {
                it('is fixed', async () => {
                    const authedClient = new __1.Etcd3(util_2.getOptions({
                        auth: {
                            username: 'connor',
                            password: 'password',
                        },
                    }));
                    await chai_1.expect(authedClient.watch().key('outside of range').create()).to.be.rejectedWith(errors_1.EtcdPermissionDeniedError);
                });
            }
        });
    });
    describe('subscription', () => {
        it('subscribes before the connection is established', async () => {
            const watcher = await client.watch().key('foo1').create();
            await expectWatching(watcher, 'foo1');
            chai_1.expect(getWatchers()).to.deep.equal([watcher]);
            await watcher.cancel();
        });
        it('subscribes while the connection is still being established', async () => {
            const watcher1 = client.watch().key('foo1').create();
            const watcher2 = client.watch().key('bar').create();
            const watchers = await Promise.all([
                watcher1.then(w => expectWatching(w, 'foo1')),
                watcher2.then(w => expectWatching(w, 'bar')),
            ]);
            chai_1.expect(getWatchers()).to.deep.equal(watchers);
            await (await watcher1).cancel();
            await (await watcher2).cancel();
        });
        it('subscribes in series', async () => {
            const watcher1 = client.watch().key('foo1').watcher();
            const watcher2 = client.watch().key('bar').watcher();
            const events = [];
            watcher1.on('connecting', () => events.push('connecting1'));
            watcher1.on('connected', () => events.push('connected1'));
            watcher2.on('connecting', () => events.push('connecting2'));
            watcher2.on('connected', () => events.push('connected2'));
            await util_1.onceEvent(watcher2, 'connected');
            chai_1.expect(events).to.deep.equal(['connecting1', 'connected1', 'connecting2', 'connected2']);
            await watcher1.cancel();
            await watcher2.cancel();
        });
        it('subscribes after the connection is fully established', async () => {
            const watcher1 = await client.watch().key('foo1').create();
            await expectWatching(watcher1, 'foo1');
            const watcher2 = await client.watch().key('bar').create();
            await expectWatching(watcher2, 'bar');
            chai_1.expect(getWatchers()).to.deep.equal([watcher1, watcher2]);
            await watcher1.cancel();
            await watcher2.cancel();
        });
        it('allows successive resubscription (issue #51)', async () => {
            const watcher1 = await client.watch().key('foo1').create();
            await expectWatching(watcher1, 'foo1');
            await watcher1.cancel();
            const watcher2 = await client.watch().key('foo1').create();
            await expectWatching(watcher2, 'foo1');
            await watcher2.cancel();
        });
    });
    describe('unsubscribing', () => {
        it('unsubscribes while the connection is established', async () => {
            const watcher = await client.watch().key('foo1').create();
            await watcher.cancel();
            await expectNotWatching(watcher, 'foo1');
            chai_1.expect(getWatchers()).to.deep.equal([]);
        });
        it('unsubscribes while the connection is being reestablished', async () => {
            await util_2.proxy.activate();
            const proxiedClient = await util_2.createTestClientAndKeys();
            const watcher = await proxiedClient.watch().key('foo1').create();
            util_2.proxy.suspend();
            await watcher.cancel();
            util_2.proxy.unsuspend();
            chai_1.expect(getWatchers()).to.deep.equal([]);
            proxiedClient.close();
            // todo: this should be awaited, but when the client is closed the tcp
            // end will time out (after 2 minutes). We can implement a client graceful
            // close once https://github.com/grpc/grpc-node/issues/1340
            util_2.proxy.deactivate();
        });
    });
});
//# sourceMappingURL=watch.test.js.map